@@ -79,18 +79,23 @@ AbstractCommand::AbstractCommand(cuid_t cuid,
                                  const SharedHandle<FileEntry>& fileEntry,
                                  RequestGroup* requestGroup,
                                  DownloadEngine* e,
-                                 const SocketHandle& s):
+                                 const SocketHandle& s,
+                                 bool incNumConnection):
   Command(cuid), checkPoint_(global::wallclock),
   timeout_(requestGroup->getTimeout()),
   requestGroup_(requestGroup),
   req_(req), fileEntry_(fileEntry), e_(e), socket_(s),
   checkSocketIsReadable_(false), checkSocketIsWritable_(false),
-  nameResolverCheck_(false)
+  nameResolverCheck_(false),
+  incNumConnection_(incNumConnection)
 {
   if(!socket_.isNull() && socket_->isOpen()) {
     setReadCheckSocket(socket_);
   }
-  requestGroup_->increaseStreamConnection();
+  if(incNumConnection_) {
+    requestGroup->increaseStreamConnection();
+  }
+  requestGroup_->increaseStreamCommand();
   requestGroup_->increaseNumCommand();
 }
 
@@ -101,7 +106,10 @@ AbstractCommand::~AbstractCommand() {
   disableNameResolverCheck(asyncNameResolver_);
 #endif // ENABLE_ASYNC_DNS
   requestGroup_->decreaseNumCommand();
-  requestGroup_->decreaseStreamConnection();
+  requestGroup_->decreaseStreamCommand();
+  if(incNumConnection_) {
+    requestGroup_->decreaseStreamConnection();
+  }
 }
 
 bool AbstractCommand::execute() {
@@ -125,26 +133,47 @@ bool AbstractCommand::execute() {
       }
       return prepareForRetry(0);
     }
-    // TODO it is not needed to check other PeerStats every time.
-    // Find faster Request when no segment is available.
-    if(!req_.isNull() && fileEntry_->countPooledRequest() > 0 &&
-       !getPieceStorage()->hasMissingUnusedPiece()) {
-      SharedHandle<Request> fasterRequest = fileEntry_->findFasterRequest(req_);
-      if(!fasterRequest.isNull()) {
-        if(getLogger()->info()) {
-          getLogger()->info("CUID#%s - Use faster Request hostname=%s, port=%u",
-                            util::itos(getCuid()).c_str(),
-                            fasterRequest->getHost().c_str(),
-                            fasterRequest->getPort());
+    if(!getPieceStorage().isNull()) {
+      segments_.clear();
+      getSegmentMan()->getInFlightSegment(segments_, getCuid());
+      if(!req_.isNull() && segments_.empty()) {
+        // This command previously has assigned segments, but it is
+        // canceled. So discard current request chain.  Plus, if no
+        // segment is available when http pipelining is used.
+        if(getLogger()->debug()) {
+          getLogger()->debug("CUID#%s - It seems previously assigned segments"
+                             " are canceled. Restart.",
+                             util::itos(getCuid()).c_str());
+        }
+        // Request::isPipeliningEnabled() == true means aria2
+        // accessed the remote server and discovered that the server
+        // supports pipelining.
+        if(!req_.isNull() && req_->isPipeliningEnabled()) {
+          e_->poolSocket(req_, createProxyRequest(), socket_);
+        }
+        return prepareForRetry(0);
+      }
+      // TODO it is not needed to check other PeerStats every time.
+      // Find faster Request when no segment is available.
+      if(!req_.isNull() && fileEntry_->countPooledRequest() > 0 &&
+         !getPieceStorage()->hasMissingUnusedPiece()) {
+        SharedHandle<Request> fasterRequest = fileEntry_->findFasterRequest(req_);
+        if(!fasterRequest.isNull()) {
+          if(getLogger()->info()) {
+            getLogger()->info("CUID#%s - Use faster Request hostname=%s, port=%u",
+                              util::itos(getCuid()).c_str(),
+                              fasterRequest->getHost().c_str(),
+                              fasterRequest->getPort());
+          }
+          // Cancel current Request object and use faster one.
+          fileEntry_->removeRequest(req_);
+          Command* command =
+            InitiateConnectionCommandFactory::createInitiateConnectionCommand
+            (getCuid(), fasterRequest, fileEntry_, requestGroup_, e_);
+          e_->setNoWait(true);
+          e_->addCommand(command);
+          return true;
         }
-        // Cancel current Request object and use faster one.
-        fileEntry_->removeRequest(req_);
-        Command* command =
-          InitiateConnectionCommandFactory::createInitiateConnectionCommand
-          (getCuid(), fasterRequest, fileEntry_, requestGroup_, e_);
-        e_->setNoWait(true);
-        e_->addCommand(command);
-        return true;
       }
     }
     if((checkSocketIsReadable_ && readEventEnabled()) ||
@@ -157,29 +186,26 @@ bool AbstractCommand::execute() {
         !nameResolverCheck_)) {
       checkPoint_ = global::wallclock;
       if(!getPieceStorage().isNull()) {
-        segments_.clear();
-        getSegmentMan()->getInFlightSegment(segments_, getCuid());
-        if(!req_.isNull() && segments_.empty()) {
-          // This command previously has assigned segments, but it is
-          // canceled. So discard current request chain.
-          if(getLogger()->debug()) {
-            getLogger()->debug("CUID#%s - It seems previously assigned segments"
-                               " are canceled. Restart.",
-                               util::itos(getCuid()).c_str());
-          }
-          return prepareForRetry(0);
-        }
         if(req_.isNull() || req_->getMaxPipelinedRequest() == 1 ||
+           // Why the following condition is necessary? That's because
+           // For single file download, SegmentMan::getSegment(cuid)
+           // is more efficient.
            getDownloadContext()->getFileEntries().size() == 1) {
-          if(segments_.empty()) {
+          size_t maxSegments = req_.isNull()?1:req_->getMaxPipelinedRequest();
+          size_t minSplitSize = calculateMinSplitSize();
+          while(segments_.size() < maxSegments) {
             SharedHandle<Segment> segment =
-              getSegmentMan()->getSegment(getCuid());
-            if(!segment.isNull()) {
+              getSegmentMan()->getSegment(getCuid(), minSplitSize);
+            if(segment.isNull()) {
+              break;
+            } else {
               segments_.push_back(segment);
             }
           }
           if(segments_.empty()) {
-            // TODO socket could be pooled here if pipelining is enabled...
+            // TODO socket could be pooled here if pipelining is
+            // enabled...  Hmm, I don't think if pipelining is enabled
+            // it does not go here.
             if(getLogger()->info()) {
               getLogger()->info(MSG_NO_SEGMENT_AVAILABLE,
                                 util::itos(getCuid()).c_str());
@@ -196,10 +222,12 @@ bool AbstractCommand::execute() {
             }
           }
         } else {
+          // For multi-file downloads
+          size_t minSplitSize = calculateMinSplitSize();
           size_t maxSegments = req_->getMaxPipelinedRequest();
           if(segments_.size() < maxSegments) {
             getSegmentMan()->getSegment
-              (segments_, getCuid(), fileEntry_, maxSegments);
+              (segments_, getCuid(), minSplitSize, fileEntry_, maxSegments);
           }
           if(segments_.empty()) {
             return prepareForRetry(0);
@@ -257,6 +285,7 @@ bool AbstractCommand::execute() {
     }
     req_->addTryCount();
     req_->resetRedirectCount();
+    req_->resetUri();
     const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
     bool isAbort = maxTries != 0 && req_->getTryCount() >= maxTries;
     if(isAbort) {
@@ -513,6 +542,7 @@ static bool isProxyRequest
   return !proxyUri.empty() && Request().setUri(proxyUri);
 }
 
+namespace {
 class DomainMatch {
 private:
   std::string hostname_;
@@ -528,6 +558,7 @@ public:
     }
   }
 };
+}
 
 static bool inNoProxy(const SharedHandle<Request>& req,
                       const std::string& noProxy)
@@ -610,7 +641,13 @@ bool AbstractCommand::isAsyncNameResolverInitialized() const
 
 void AbstractCommand::initAsyncNameResolver(const std::string& hostname)
 {
-  asyncNameResolver_.reset(new AsyncNameResolver());
+  int family;
+  if(getOption()->getAsBool(PREF_ENABLE_ASYNC_DNS6)) {
+    family = AF_UNSPEC;
+  } else {
+    family = AF_INET;
+  }
+  asyncNameResolver_.reset(new AsyncNameResolver(family));
   if(getLogger()->info()) {
     getLogger()->info(MSG_RESOLVING_HOSTNAME,
                       util::itos(getCuid()).c_str(), hostname.c_str());
@@ -673,6 +710,7 @@ std::string AbstractCommand::resolveHostname
 (std::vector<std::string>& addrs, const std::string& hostname, uint16_t port)
 {
   if(util::isNumericHost(hostname)) {
+    addrs.push_back(hostname);
     return hostname;
   }
   e_->findAllCachedIPAddresses(std::back_inserter(addrs), hostname, port);
@@ -812,4 +850,13 @@ void AbstractCommand::createSocket()
   socket_.reset(new SocketCore());
 }
 
+size_t AbstractCommand::calculateMinSplitSize() const
+{
+  if(!req_.isNull() && req_->isPipeliningEnabled()) {
+    return getDownloadContext()->getPieceLength();
+  } else {
+    return getOption()->getAsInt(PREF_MIN_SPLIT_SIZE);
+  }
+}
+
 } // namespace aria2
