@@ -39,11 +39,10 @@
 
 #include <map>
 
-#include "TimeA2.h"
+#include "TimerA2.h"
 
 namespace aria2 {
 
-class Option;
 class Logger;
 class BtRuntime;
 class BtSeederStateChoke;
@@ -52,26 +51,28 @@ class PieceStorage;
 
 class DefaultPeerStorage : public PeerStorage {
 private:
-  SharedHandle<BtRuntime> _btRuntime;
-  SharedHandle<PieceStorage> _pieceStorage;
-  const Option* option;
-  std::deque<SharedHandle<Peer> > peers;
-  Logger* logger;
-  uint64_t removedPeerSessionDownloadLength;
-  uint64_t removedPeerSessionUploadLength;
+  SharedHandle<BtRuntime> btRuntime_;
+  SharedHandle<PieceStorage> pieceStorage_;
+  std::deque<SharedHandle<Peer> > peers_;
+  std::deque<SharedHandle<Peer> > droppedPeers_;
+  Logger* logger_;
+  uint64_t removedPeerSessionDownloadLength_;
+  uint64_t removedPeerSessionUploadLength_;
 
-  BtSeederStateChoke* _seederStateChoke;
-  BtLeecherStateChoke* _leecherStateChoke;
+  BtSeederStateChoke* seederStateChoke_;
+  BtLeecherStateChoke* leecherStateChoke_;
 
-  std::map<std::string, TransferStat> _peerTransferStatMap;
+  std::map<std::string, TransferStat> peerTransferStatMap_;
 
-  Time _lastTransferStatMapUpdated;
+  Timer lastTransferStatMapUpdated_;
 
-  TransferStat _cachedTransferStat;
+  TransferStat cachedTransferStat_;
 
   bool isPeerAlreadyAdded(const SharedHandle<Peer>& peer);
+
+  void addDroppedPeer(const SharedHandle<Peer>& peer);
 public:
-  DefaultPeerStorage(const Option* option);
+  DefaultPeerStorage();
 
   virtual ~DefaultPeerStorage();
 
@@ -87,6 +88,8 @@ public:
 
   virtual const std::deque<SharedHandle<Peer> >& getPeers();
 
+  virtual const std::deque<SharedHandle<Peer> >& getDroppedPeers();
+
   virtual bool isPeerAvailable();
 
   virtual void getActivePeers(std::vector<SharedHandle<Peer> >& peers);
