@@ -43,19 +43,17 @@ namespace aria2 {
 
 class IndexBtMessageValidator : public BtMessageValidator {
 private:
-  const IndexBtMessage* _message;
-  size_t _numPiece;
+  const IndexBtMessage* message_;
+  size_t numPiece_;
 public:
   IndexBtMessageValidator(const IndexBtMessage* message,
                           size_t numPiece):
-    _message(message),
-    _numPiece(numPiece) {}
+    message_(message),
+    numPiece_(numPiece) {}
 
-  virtual bool validate(Errors& errors)
+  virtual void validate()
   {
-    // TODO
-    bittorrent::checkIndex(_message->getIndex(), _numPiece);
-    return true;
+    bittorrent::checkIndex(message_->getIndex(), numPiece_);
   }
 
 };
